@@ -1,6 +1,6 @@
-/* $Id: exo-icon-bar.c 40 2004-12-06 20:46:55Z bmeurer $ */
+/* $Id$ */
 /*-
- * Copyright (c) 2004 os-cillation e.K.
+ * Copyright (c) 2004-2006 os-cillation e.K.
  *
  * Written by Benedikt Meurer <benny@xfce.org>.
  *
@@ -24,10 +24,12 @@
 #include <config.h>
 #endif
 
-#include <libxfce4util/libxfce4util.h>
-
+#include <exo/exo-config.h>
 #include <exo/exo-icon-bar.h>
 #include <exo/exo-marshal.h>
+#include <exo/exo-private.h>
+#include <exo/exo-string.h>
+#include <exo/exo-alias.h>
 
 
 
@@ -78,7 +80,6 @@ static void            exo_icon_bar_style_set             (GtkWidget        *wid
                                                            GtkStyle         *previous_style);
 static void            exo_icon_bar_realize               (GtkWidget        *widget);
 static void            exo_icon_bar_unrealize             (GtkWidget        *widget);
-static void            exo_icon_bar_map                   (GtkWidget        *widget);
 static void            exo_icon_bar_size_request          (GtkWidget        *widget,
                                                            GtkRequisition   *requisition);
 static void            exo_icon_bar_size_allocate         (GtkWidget        *widget,
@@ -178,12 +179,28 @@ struct _ExoIconBarPrivate
 
 
 
-static GObjectClass *parent_class;
-static guint icon_bar_signals[LAST_SIGNAL];
+static GObjectClass *exo_icon_bar_parent_class;
+static guint         icon_bar_signals[LAST_SIGNAL];
 
 
 
-G_DEFINE_TYPE (ExoIconBar, exo_icon_bar, GTK_TYPE_CONTAINER);
+GType
+exo_icon_bar_get_type (void)
+{
+  static GType type = G_TYPE_INVALID;
+
+  if (G_UNLIKELY (type == G_TYPE_INVALID))
+    {
+      type = _exo_g_type_register_simple (GTK_TYPE_CONTAINER,
+                                          "ExoIconBar",
+                                          sizeof (ExoIconBarClass),
+                                          exo_icon_bar_class_init,
+                                          sizeof (ExoIconBar),
+                                          exo_icon_bar_init);
+    }
+
+  return type;
+}
 
 
 
@@ -196,7 +213,7 @@ exo_icon_bar_class_init (ExoIconBarClass *klass)
 
   g_type_class_add_private (klass, sizeof (ExoIconBarPrivate));
 
-  parent_class = g_type_class_peek_parent (klass);
+  exo_icon_bar_parent_class = g_type_class_peek_parent (klass);
 
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = exo_icon_bar_finalize;
@@ -210,7 +227,6 @@ exo_icon_bar_class_init (ExoIconBarClass *klass)
   gtkwidget_class->style_set = exo_icon_bar_style_set;
   gtkwidget_class->realize = exo_icon_bar_realize;
   gtkwidget_class->unrealize = exo_icon_bar_unrealize;
-  gtkwidget_class->map = exo_icon_bar_map;
   gtkwidget_class->size_request = exo_icon_bar_size_request;
   gtkwidget_class->size_allocate = exo_icon_bar_size_allocate;
   gtkwidget_class->expose_event = exo_icon_bar_expose;
@@ -234,7 +250,7 @@ exo_icon_bar_class_init (ExoIconBarClass *klass)
                                                       _("The orientation of the iconbar"),
                                                       GTK_TYPE_ORIENTATION,
                                                       GTK_ORIENTATION_VERTICAL,
-                                                      G_PARAM_READWRITE));
+                                                      EXO_PARAM_READWRITE));
 
   /**
    * ExoIconBar:pixbuf-column:
@@ -250,7 +266,7 @@ exo_icon_bar_class_init (ExoIconBarClass *klass)
                                                      _("Pixbuf column"),
                                                      _("Model column used to retrieve the icon pixbuf from"),
                                                      -1, G_MAXINT, -1,
-                                                     G_PARAM_READWRITE));
+                                                     EXO_PARAM_READWRITE));
 
   /**
    * ExoIconBar:text-column:
@@ -266,7 +282,7 @@ exo_icon_bar_class_init (ExoIconBarClass *klass)
                                                      _("Text column"),
                                                      _("Model column used to retrieve the text from"),
                                                      -1, G_MAXINT, -1,
-                                                     G_PARAM_READWRITE));
+                                                     EXO_PARAM_READWRITE));
 
   /**
    * ExoIconBar:model:
@@ -279,7 +295,7 @@ exo_icon_bar_class_init (ExoIconBarClass *klass)
                                                         _("Icon Bar Model"),
                                                         _("Model for the icon bar"),
                                                         GTK_TYPE_TREE_MODEL,
-                                                        G_PARAM_READWRITE));
+                                                        EXO_PARAM_READWRITE));
 
   /**
    * ExoIconBar:active:
@@ -296,49 +312,49 @@ exo_icon_bar_class_init (ExoIconBarClass *klass)
                                                      _("Active"),
                                                      _("Active item index"),
                                                      -1, G_MAXINT, -1,
-                                                     G_PARAM_READWRITE));
+                                                     EXO_PARAM_READWRITE));
 
   gtk_widget_class_install_style_property (gtkwidget_class,
                                            g_param_spec_boxed ("active-item-fill-color",
                                                                _("Active item fill color"),
                                                                _("Active item fill color"),
                                                                GDK_TYPE_COLOR,
-                                                               G_PARAM_READABLE));
+                                                               EXO_PARAM_READABLE));
 
   gtk_widget_class_install_style_property (gtkwidget_class,
                                            g_param_spec_boxed ("active-item-border-color",
                                                                _("Active item border color"),
                                                                _("Active item border color"),
                                                                GDK_TYPE_COLOR,
-                                                               G_PARAM_READABLE));
+                                                               EXO_PARAM_READABLE));
 
   gtk_widget_class_install_style_property (gtkwidget_class,
                                            g_param_spec_boxed ("active-item-text-color",
                                                                _("Active item text color"),
                                                                _("Active item text color"),
                                                                GDK_TYPE_COLOR,
-                                                               G_PARAM_READABLE));
+                                                               EXO_PARAM_READABLE));
 
   gtk_widget_class_install_style_property (gtkwidget_class,
                                            g_param_spec_boxed ("cursor-item-fill-color",
                                                                _("Cursor item fill color"),
                                                                _("Cursor item fill color"),
                                                                GDK_TYPE_COLOR,
-                                                               G_PARAM_READABLE));
+                                                               EXO_PARAM_READABLE));
 
   gtk_widget_class_install_style_property (gtkwidget_class,
                                            g_param_spec_boxed ("cursor-item-border-color",
                                                                _("Cursor item border color"),
                                                                _("Cursor item border color"),
                                                                GDK_TYPE_COLOR,
-                                                               G_PARAM_READABLE));
+                                                               EXO_PARAM_READABLE));
 
   gtk_widget_class_install_style_property (gtkwidget_class,
                                            g_param_spec_boxed ("cursor-item-text-color",
                                                                _("Cursor item text color"),
                                                                _("Cursor item text color"),
                                                                GDK_TYPE_COLOR,
-                                                               G_PARAM_READABLE));
+                                                               EXO_PARAM_READABLE));
 
   /**
    * ExoIconBar::set-scroll-adjustments:
@@ -349,7 +365,7 @@ exo_icon_bar_class_init (ExoIconBarClass *klass)
    * Used internally to make the #ExoIconBar scrollable.
    **/
   gtkwidget_class->set_scroll_adjustments_signal =
-    g_signal_new ("set-scroll-adjustments",
+    g_signal_new (I_("set-scroll-adjustments"),
                   G_TYPE_FROM_CLASS (gobject_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ExoIconBarClass, set_scroll_adjustments),
@@ -367,7 +383,7 @@ exo_icon_bar_class_init (ExoIconBarClass *klass)
    * changes.
    **/
   icon_bar_signals[SELECTION_CHANGED] =
-    g_signal_new ("selection-changed",
+    g_signal_new (I_("selection-changed"),
                   G_TYPE_FROM_CLASS (gobject_class),
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (ExoIconBarClass, selection_changed),
@@ -404,7 +420,7 @@ exo_icon_bar_destroy (GtkObject *object)
 
   exo_icon_bar_set_model (icon_bar, NULL);
 
-  GTK_OBJECT_CLASS (parent_class)->destroy (object);
+  (*GTK_OBJECT_CLASS (exo_icon_bar_parent_class)->destroy) (object);
 }
 
 
@@ -416,7 +432,7 @@ exo_icon_bar_finalize (GObject *object)
 
   g_object_unref (G_OBJECT (icon_bar->priv->layout));
 
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  (*G_OBJECT_CLASS (exo_icon_bar_parent_class)->finalize) (object);
 }
 
 
@@ -503,7 +519,7 @@ exo_icon_bar_style_set (GtkWidget *widget,
 {
   ExoIconBar *icon_bar = EXO_ICON_BAR (widget);
 
-  GTK_WIDGET_CLASS (parent_class)->style_set (widget, previous_style);
+  (*GTK_WIDGET_CLASS (exo_icon_bar_parent_class)->style_set) (widget, previous_style);
 
   if (GTK_WIDGET_REALIZED (widget))
     {
@@ -560,6 +576,7 @@ exo_icon_bar_realize (GtkWidget *widget)
   widget->style = gtk_style_attach (widget->style, widget->window);
   gdk_window_set_background (widget->window, &widget->style->base[widget->state]);
   gdk_window_set_background (icon_bar->priv->bin_window, &widget->style->base[widget->state]);
+  gdk_window_show (icon_bar->priv->bin_window);
 }
 
 
@@ -574,20 +591,7 @@ exo_icon_bar_unrealize (GtkWidget *widget)
   icon_bar->priv->bin_window = NULL;
 
   /* GtkWidget::unrealize destroys children and widget->window */
-  GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
-}
-
-
-
-static void
-exo_icon_bar_map (GtkWidget *widget)
-{
-  ExoIconBar *icon_bar = EXO_ICON_BAR (widget);
-
-  GTK_WIDGET_SET_FLAGS (widget, GTK_MAPPED);
-
-  gdk_window_show (icon_bar->priv->bin_window);
-  gdk_window_show (widget->window);
+  (*GTK_WIDGET_CLASS (exo_icon_bar_parent_class)->unrealize) (widget);
 }
 
 
@@ -956,9 +960,9 @@ exo_icon_bar_paint_item (ExoIconBar     *icon_bar,
       y = icon_bar->priv->item_height * item->index;
 
       px = (icon_bar->priv->item_width - item->pixbuf_width) / 2 + focus_pad + focus_width;
-      py = (icon_bar->priv->item_height - (item->pixbuf_height + item->layout_height + 2 * ICON_TEXT_PADDING)) / 2
+      py = (icon_bar->priv->item_height - (item->pixbuf_height + item->layout_height + ICON_TEXT_PADDING)) / 2
          + icon_bar->priv->item_height * item->index + focus_pad + focus_width;
-      lx = (icon_bar->priv->item_width - (item->layout_width)) / 2 + focus_pad;
+      lx = (icon_bar->priv->item_width - (item->layout_width + ICON_TEXT_PADDING)) / 2 + focus_pad;
       ly = py + item->pixbuf_height + ICON_TEXT_PADDING;
     }
   else
@@ -1171,7 +1175,7 @@ exo_icon_bar_calculate_item_size (ExoIconBar      *icon_bar,
       item->layout_height = 0;
     }
 
-  item->width = MAX (item->layout_width + 2 * ICON_TEXT_PADDING, item->pixbuf_width)
+  item->width = MAX (item->layout_width, item->pixbuf_width) + 2 * ICON_TEXT_PADDING
               + 2 * (focus_width + focus_pad);
   item->height = item->layout_height + 2 * (focus_width + focus_pad + ICON_TEXT_PADDING)
                + item->pixbuf_height;
@@ -1240,7 +1244,7 @@ exo_icon_bar_item_new (void)
 {
   ExoIconBarItem *item;
 
-  item = g_new0 (ExoIconBarItem, 1);
+  item = _exo_slice_new0 (ExoIconBarItem);
   item->width = -1;
   item->height = -1;
 
@@ -1252,7 +1256,7 @@ exo_icon_bar_item_new (void)
 static void
 exo_icon_bar_item_free (ExoIconBarItem *item)
 {
-  g_free (item);
+  _exo_slice_free (ExoIconBarItem, item);
 }
 
 
@@ -1400,25 +1404,22 @@ exo_icon_bar_rows_reordered (GtkTreeModel *model,
   gint             i;
 
   length = gtk_tree_model_iter_n_children (model, NULL);
-  inverted_order = g_new (gint, length);
+  inverted_order = g_newa (gint, length);
 
   /* invert the array */
   for (i = 0; i < length; ++i)
     inverted_order[new_order[i]] = i;
 
-  item_array = g_new (ExoIconBarItem *, length);
+  item_array = g_newa (ExoIconBarItem *, length);
   for (i = 0, lp = icon_bar->priv->items; lp != NULL; ++i, lp = lp->next)
     item_array[inverted_order[i]] = lp->data;
 
-  g_free (inverted_order);
-
   for (i = 0; i < length; ++i)
     {
       item_array[i]->index = i;
       items = g_list_append (items, item_array[i]);
     }
 
-  g_free (item_array);
   g_list_free (icon_bar->priv->items);
   icon_bar->priv->items = g_list_reverse (items);
 
@@ -1852,3 +1853,8 @@ exo_icon_bar_set_active_iter (ExoIconBar  *icon_bar,
       gtk_tree_path_free (path);
     }
 }
+
+
+
+#define __EXO_ICON_BAR_C__
+#include <exo/exo-aliasdef.c>
